"""ORCA descriptors library for QSAR analysis."""

from orca_descriptors.orca import Orca
from orca_descriptors.batch_processing import ORCABatchProcessing

__all__ = ["Orca", "ORCABatchProcessing"]
__version__ = "0.3.3b2"

